@@ -31,6 +31,7 @@ import jujuclient
 
 from quickstart import (
     juju,
+    platform_support,
     settings,
     ssh,
     utils,
@@ -57,58 +58,59 @@ class ProgramExit(Exception):
         return b'juju-quickstart: error: {}'.format(self.message)
 
 
-def ensure_dependencies(distro_only):
+def ensure_dependencies(distro_only, platform, juju_command):
     """Ensure that Juju and LXC are installed.
 
-    If the "juju" command is not found in the PATH, then install and setup
-    Juju, including the packages required to bootstrap local environments.
-    This is usually done by adding the Juju stable PPA and installing the
-    juju-core and juju-local packages.
+    If the "juju" command is not found in the PATH, then install and
+    setup Juju.  For Linux with apt, this includes the packages required
+    to bootstrap local environments and is usually done by adding the
+    Juju stable PPA and installing the juju-core and juju-local
+    packages.
 
     If distro_only is True, the above PPA is not added to the apt sources, and
     we assume Juju packages are already available in the distro repository.
+    distro_only is ignored for host other than Linux with apt.
 
     Return the Juju version tuple when Juju is available.
 
     Raise a ProgramExit if an error occurs installing packages or retrieving
     the Juju version.
     """
+    try:
+        installer = platform_support.get_juju_installer(platform)
+    except ValueError as err:
+        raise ProgramExit(bytes(err))
+
     required_packages = []
     # Check if juju is installed.
     try:
-        juju_version = utils.get_juju_version()
+        juju_version = utils.get_juju_version(juju_command)
     except ValueError:
         # Juju is not installed or configured properly. To ensure everything
         # is set up correctly, also install packages required to run
         # environments using the local provider.
-        required_packages.extend(['juju-core', 'juju-local'])
+        required_packages.extend(settings.JUJU_PACKAGES.get(platform))
         juju_version = None
     else:
-        # Check if LXC is installed.
-        retcode = utils.call('/usr/bin/lxc-ls')[0]
-        if retcode:
-            # Some packages (e.g. lxc and mongodb-server) are required to
-            # support bootstrapping environments using the local provider.
-            # All those packages are installed as juju-local dependencies.
-            required_packages.append('juju-local')
+        if platform_support.supports_local(platform):
+            # Check if LXC is installed.
+            retcode = utils.call('/usr/bin/lxc-ls')[0]
+            if retcode:
+                # Some packages (e.g. lxc and mongodb-server) are required to
+                # support bootstrapping environments using the local provider.
+                # All those packages are installed as juju-local dependencies.
+                required_packages.append('juju-local')
     if required_packages:
-        if not distro_only:
-            try:
-                utils.add_apt_repository('ppa:juju/stable')
-            except OSError as err:
-                raise ProgramExit(bytes(err))
-        print('sudo privileges will be used for the installation of \n'
-              'the following packages: {}\n'
-              'this can take a while...'.format(', '.join(required_packages)))
-        retcode, _, error = utils.call(
-            'sudo', '/usr/bin/apt-get', 'install', '-y', *required_packages)
-        if retcode:
-            raise ProgramExit(error)
+        try:
+            installer(distro_only, required_packages)
+        except OSError as error:
+            raise ProgramExit(bytes(error))
+
     # Return the current Juju version.
     if juju_version is None:
         # Juju has been just installed, retrieve its version.
         try:
-            juju_version = utils.get_juju_version()
+            juju_version = utils.get_juju_version(juju_command)
         except ValueError as err:
             raise ProgramExit(bytes(err))
     return juju_version
@@ -156,13 +158,13 @@ def ensure_ssh_keys():
                 b'\nIf you would like to create the keys yourself,\n'
                 b'please run this command, follow its instructions,\n'
                 b'and then re-run quickstart:\n'
-                b'  ssh-keygen -b 4096 -t rsa'
-            )
+                b'  ssh-keygen -b 4096 -t rsa')
     except OSError as err:
         raise ProgramExit(bytes(err))
 
 
-def bootstrap(env_name, requires_sudo=False, debug=False):
+def bootstrap(env_name, juju_command, requires_sudo=False, debug=False,
+              upload_tools=False, upload_series=None, constraints=None):
     """Bootstrap the Juju environment with the given name.
 
     Do not bootstrap the environment if already bootstrapped.
@@ -182,11 +184,17 @@ def bootstrap(env_name, requires_sudo=False, debug=False):
     Raise a ProgramExit if any error occurs in the bootstrap process.
     """
     already_bootstrapped = False
-    cmd = [settings.JUJU_CMD, 'bootstrap', '-e', env_name]
+    cmd = [juju_command, 'bootstrap', '-e', env_name]
     if requires_sudo:
         cmd.insert(0, 'sudo')
     if debug:
         cmd.append('--debug')
+    if upload_tools:
+        cmd.append('--upload-tools')
+    if upload_series is not None:
+        cmd.extend(['--upload-series', upload_series])
+    if constraints is not None:
+        cmd.extend(['--constraints', constraints])
     retcode, _, error = utils.call(*cmd)
     if retcode:
         # XXX frankban 2013-11-13 bug 1252322: the check below is weak. We are
@@ -219,7 +227,7 @@ def bootstrap(env_name, requires_sudo=False, debug=False):
     timeout = time.time() + (60*10)
     while time.time() < timeout:
         retcode, output, error = utils.call(
-            settings.JUJU_CMD, 'status', '-e', env_name, '--format', 'yaml')
+            juju_command, 'status', '-e', env_name, '--format', 'yaml')
         if retcode:
             continue
         # Ensure the state server is up and the agent is started.
@@ -238,29 +246,30 @@ def bootstrap(env_name, requires_sudo=False, debug=False):
     raise ProgramExit('the state server is not ready:\n{}'.format(details))
 
 
-def get_admin_secret(env_name, juju_home):
-    """Read the admin-secret from the generated environment file.
+def get_value_from_jenv(env_name, juju_home, key):
+    """Read the key from the generated environment file.
+
+    At bootstrap, juju (v1.16 and later) writes a generated file
+    located in JUJU_HOME.  Return the value for the key.
 
-    At bootstrap, juju (v1.16 and later) writes the admin-secret to a
-    generated file located in JUJU_HOME.  Return the value.
     Raise a ValueError if:
         - the environment file is not found;
         - the environment file contents are not parsable by YAML;
         - the YAML contents are not properly structured;
-        - the admin-secret is not found.
+        - the key is not found.
     """
     filename = '{}.jenv'.format(env_name)
     juju_env_file = os.path.expanduser(
         os.path.join(juju_home, 'environments', filename))
     jenv_db = envs.load_generated(juju_env_file)
     try:
-        return jenv_db['admin-secret']
+        return jenv_db[key]
     except KeyError:
-        msg = 'admin-secret not found in {}'.format(juju_env_file)
+        msg = '{} not found in {}'.format(key, juju_env_file)
         raise ValueError(msg.encode('utf-8'))
 
 
-def get_api_url(env_name):
+def get_api_url(env_name, juju_command):
     """Return a Juju API URL for the given environment name.
 
     Use the Juju CLI in a subprocess in order to retrieve the API addresses.
@@ -268,7 +277,7 @@ def get_api_url(env_name):
     Raise a ProgramExit if any error occurs.
     """
     retcode, output, error = utils.call(
-        settings.JUJU_CMD, 'api-endpoints', '-e', env_name, '--format', 'json')
+        juju_command, 'api-endpoints', '-e', env_name, '--format', 'json')
     if retcode:
         raise ProgramExit(error)
     # Assuming there is always at least one API address, grab the first one
@@ -323,27 +332,41 @@ def create_auth_token(env):
     return result['Token']
 
 
-def deploy_gui(
-        env, service_name, machine, charm_url=None, check_preexisting=False):
-    """Deploy and expose the given service, reusing the bootstrap node.
+def check_environment(
+        env, service_name, charm_url, env_type, bootstrap_node_series,
+        check_preexisting):
+    """Inspect the current Juju environment.
 
-    Only deploy the service if not already present in the environment.
-    Do not add a unit to the service if the unit is already there.
+    This function is responsible for retrieving all the information required
+    to deploy the Juju GUI charm to the current environment.
 
-    Receive an authenticated Juju Environment instance, the name of the
-    service, the machine where to deploy to (or None for a new machine),
-    the optional Juju GUI charm URL (e.g. cs:~juju-gui/precise/juju-gui-42),
-    and a flag (check_preexisting) that can be set to True in order to make
-    the function check for a pre-existing service and/or unit.
+    Receive the following arguments:
+        - env: an authenticated Juju Environment instance;
+        - service_name: the name of the service to look for;
+        - charm_url: a fully qualified charm URL if the user requested the
+          deployment of a custom charm, or None otherwise;
+        - env_type: the environment's provider type;
+        - bootstrap_node_series: the bootstrap node series;
+        - check_preexisting: whether to check for a pre-existing service and/or
+          unit.
 
     If the charm URL is not provided, and the service is not already deployed,
-    the function tries to retrieve it from charmworld. In this case a default
-    charm URL is used if charmworld is not available.
+    the function tries to retrieve it from the charmworld API. In this case a
+    default charm URL is used if charmworld is not available.
+
+    Return a tuple including the following values:
+        - charm_url: the charm URL that will be used to deploy the service;
+        - machine: the machine where to deploy to (e.g. "0") or None if a new
+          machine must be created;
+        - service_data: the service info as returned by the mega-watcher for
+          services, or None if the service is not present in the environment;
+        - unit_data: the unit info as returned by the mega-watcher for units,
+          or None if the unit is not present in the environment.
 
-    Return the name of the first running unit belonging to the given service.
     Raise a ProgramExit if the API server returns an error response.
     """
-    service_data, unit_data = None, None
+    print('bootstrap node series: {}'.format(bootstrap_node_series))
+    service_data, unit_data, machine = None, None, None
     if check_preexisting:
         # The service and/or the unit can be already in the environment.
         try:
@@ -353,16 +376,62 @@ def deploy_gui(
         service_data, unit_data = utils.get_service_info(status, service_name)
     if service_data is None:
         # The service does not exist in the environment.
-        print('requesting {} deployment'.format(service_name))
         if charm_url is None:
+            # The user did not provide a customized charm.
+            if bootstrap_node_series in settings.JUJU_GUI_SUPPORTED_SERIES:
+                series = bootstrap_node_series
+            else:
+                series = settings.JUJU_GUI_SUPPORTED_SERIES[-1]
             try:
-                charm_url = utils.get_charm_url()
+                # Try to get the charm URL from charmworld.
+                charm_url = utils.get_charm_url(series)
             except (IOError, ValueError) as err:
+                # Fall back to the default URL for the current series.
                 msg = 'unable to retrieve the {} charm URL from the API: {}'
                 logging.warn(msg.format(service_name, err))
-                charm_url = settings.DEFAULT_CHARM_URL
-        utils.check_gui_charm_url(charm_url)
-        # Deploy the service without units.
+                charm_url = settings.DEFAULT_CHARM_URLS[series]
+    else:
+        # A deployed service already exists in the environment: ignore the
+        # provided charm URL and just use the already deployed charm.
+        charm_url = service_data['CharmURL']
+    charm = utils.parse_gui_charm_url(charm_url)
+    # Deploy on the bootstrap node if the following conditions are satisfied:
+    # - we are not using the local provider (which uses localhost);
+    # - we are not using the azure provider (in which availability sets prevent
+    #   us from relying on the bootstrap node);
+    # - the requested charm and the bootstrap node have the same series.
+    if (
+        (env_type not in ('local', 'azure')) and
+        (charm.series == bootstrap_node_series)
+    ):
+        machine = '0'
+    return charm_url, machine, service_data, unit_data
+
+
+def deploy_gui(env, service_name, charm_url, machine, service_data, unit_data):
+    """Deploy and expose the given service to the given machine.
+
+    Only deploy the service if not already present in the environment.
+    Do not add a unit to the service if the unit is already there.
+
+    Receive the following arguments:
+        - env: an authenticated Juju Environment instance;
+        - service_name: the name of the service to be deployed;
+        - charm_url: the fully qualified charm URL to be used to deploy the
+          service;
+        - machine: the machine where to deploy to (e.g. "0") or None if a new
+          machine must be created;
+        - service_data: the service info as returned by the mega-watcher for
+          services, or None if the service is not present in the environment;
+        - unit_data: the unit info as returned by the mega-watcher for units,
+          or None if the unit is not present in the environment.
+
+    Return the name of the first running unit belonging to the given service.
+    Raise a ProgramExit if the API server returns an error response.
+    """
+    if service_data is None:
+        # The service is not in the environment: deploy it without units.
+        print('requesting {} deployment'.format(service_name))
         try:
             env.deploy(service_name, charm_url, num_units=0)
         except jujuclient.EnvError as err:
@@ -372,7 +441,6 @@ def deploy_gui(
     else:
         # We already have the service in the environment.
         print('service {} already deployed'.format(service_name))
-        utils.check_gui_charm_url(service_data['CharmURL'])
         service_exposed = service_data.get('Exposed', False)
     # At this point the service is surely deployed in the environment: expose
     # it if necessary and add a unit if it is missing.
