@@ -102,7 +102,10 @@ import operator
 
 import urwid
 
-from quickstart import settings
+from quickstart import (
+    platform_support,
+    settings,
+)
 from quickstart.cli import (
     base,
     forms,
@@ -170,6 +173,9 @@ def env_index(app, env_type_db, env_db, save_callable):
         # Use the newly created environment.
         raise ui.AppExit((env_db, env_data))
 
+    platform = platform_support.get_platform()
+    supports_local = platform_support.supports_local(platform)
+
     if environments:
         title = 'Select an existing Juju environment or create a new one'
         widgets = [
@@ -188,16 +194,24 @@ def env_index(app, env_type_db, env_db, save_callable):
                 'Quickstart in interactive mode again by passing the -i flag, '
                 'e.g.:\n',
                 ('highlight', '$ juju quickstart -i'),
-                '\n\nAt the bottom of the page you can find links to manually '
-                'create new environments. If you instead prefer to quickly '
-                'start your Juju experience in a local environment (LXC), '
-                'just click the link below:'
             ]),
-            urwid.Divider(),
-            ui.MenuButton(
-                '\N{BULLET} automatically create and bootstrap a local '
-                'environment', ui.thunk(create_and_start_local_env)),
         ]
+        # If the current platform supports local Juju environments, add an
+        # option to automatically create and bootstrap one.
+        if supports_local:
+            widgets.extend([
+                urwid.Text([
+                    '\nAt the bottom of the page you can find links to '
+                    'manually create new environments. If you instead prefer '
+                    'to quickly start your Juju experience in a local '
+                    'environment (LXC), just click the link below:'
+                ]),
+                urwid.Divider(),
+                ui.MenuButton(
+                    '\N{BULLET} automatically create and bootstrap a local '
+                    'environment', ui.thunk(create_and_start_local_env)),
+            ])
+
     app.set_title(title)
     # Start creating the page contents: a list of selectable environments.
     # Wouldn't it be nice if we were able to highlight in some way the
@@ -225,7 +239,8 @@ def env_index(app, env_type_db, env_db, save_callable):
         env_short_description = envs.get_env_short_description(env_data)
         text = [bullet, ' {}'.format(env_short_description)]
         widgets.append(ui.MenuButton(text, ui.thunk(detail_view, env_data)))
-    # Add the buttons used to create new environments.
+
+    # Set up the "create a new environment" section.
     widgets.extend([
         urwid.Divider(),
         urwid.Text((
@@ -233,16 +248,26 @@ def env_index(app, env_type_db, env_db, save_callable):
         urwid.Divider(),
     ])
     # The Juju GUI can be safely installed in the bootstrap node only if its
-    # series is "precise". Suggest this setting by pre-filling the value.
-    preferred_series = settings.JUJU_GUI_PREFERRED_SERIES
+    # series matches one of the series supported by the GUI.
+    # Suggest the most recent supported series by pre-filling the value.
+    preferred_series = settings.JUJU_GUI_SUPPORTED_SERIES[-1]
+    # Retrieve the list of supported environment types: exclude the local
+    # environment if not supported by the current OS platform.
+    filter_function = None
+    if not supports_local:
+        filter_function = lambda env_type, _: env_type != 'local'
+    supported_env_types = envs.get_supported_env_types(
+        env_type_db, filter_function=filter_function)
+    # Add the buttons used to create new environments.
     widgets.extend([
         ui.MenuButton(
             ['\N{BULLET} new ', ('highlight', label), ' environment'],
             ui.thunk(edit_view, {
                 'type': env_type, 'default-series': preferred_series})
         )
-        for env_type, label in envs.get_supported_env_types(env_type_db)
+        for env_type, label in supported_env_types
     ])
+
     # Set up the application status messages.
     status = [' \N{UPWARDS ARROW LEFTWARDS OF DOWNWARDS ARROW} navigate ']
     if default_found:
