@@ -33,6 +33,7 @@ import quickstart
 from quickstart import (
     app,
     packaging,
+    platform_support,
     settings,
     utils,
 )
@@ -54,7 +55,7 @@ class _DescriptionAction(argparse.Action):
         parser.exit()
 
 
-def _get_packaging_info(juju_source):
+def _get_packaging_info(juju_source, platform):
     """Return packaging info based on the given juju source.
 
     The juju_source argument can be either "ppa" or "distro".
@@ -65,16 +66,23 @@ def _get_packaging_info(juju_source):
         - distro_only_help: the help text for the --distro-only flag;
         - ppa_help: the help text for the --ppa flag.
     """
-    distro_only_help = ('Do not use external sources when installing and '
-                        'setting up Juju')
-    ppa_help = 'Use external sources when installing and setting up Juju'
-    disable_help = '\n(enabled by default, use {} to disable)'
-    if juju_source == 'distro':
-        distro_only = True
-        distro_only_help += disable_help.format('--ppa')
+    if platform == settings.LINUX_APT:
+        distro_only_help = ('Do not use external sources when installing and '
+                            'setting up Juju')
+        ppa_help = 'Use external sources when installing and setting up Juju'
+        disable_help = '\n(enabled by default, use {} to disable)'
+        if juju_source == 'distro':
+            distro_only = True
+            distro_only_help += disable_help.format('--ppa')
+        else:
+            distro_only = False
+            ppa_help += disable_help.format('--distro-only')
     else:
-        distro_only = False
-        ppa_help += disable_help.format('--distro-only')
+        # If not LINUX_APT then distro_only and ppa make no sense.  Prevent
+        # the user from seeing them but allow reasonable values in options.
+        distro_only = True
+        distro_only_help = argparse.SUPPRESS
+        ppa_help = argparse.SUPPRESS
     return distro_only, distro_only_help, ppa_help
 
 
@@ -153,10 +161,9 @@ def _validate_charm_url(options, parser):
         # The user requested a bundle deployment.
         options.bundle and
         # This is the official Juju GUI charm.
-        charm.name == settings.JUJU_GUI_CHARM_NAME and
-        not charm.user and
+        charm.name == settings.JUJU_GUI_CHARM_NAME and not charm.user and
         # The charm at this revision does not support bundle deployments.
-        charm.revision < settings.MINIMUM_CHARM_REVISION_FOR_BUNDLES
+        charm.revision < settings.MINIMUM_REVISIONS_FOR_BUNDLES[charm.series]
     ):
         return parser.error(
             'bundle deployments not supported by the requested charm '
@@ -278,11 +285,17 @@ def _setup_env(options, parser):
         # This is a non-interactive session and we need to validate the
         # selected environment before proceeding.
         env_data = _retrieve_env_data(parser, env_type_db, env_db, env_name)
+    # Check for local support, if requested.
+    options.env_type = env_data['type']
+    no_local_support = not platform_support.supports_local(options.platform)
+    if options.env_type == 'local' and no_local_support:
+        return parser.error(
+            'this host platform does not support local environments'
+        )
     # Update the options namespace with the new values.
     options.admin_secret = env_data.get('admin-secret')
     options.env_file = env_file
     options.env_name = env_data['name']
-    options.env_type = env_data['type']
     options.default_series = env_data.get('default-series')
     options.interactive = interactive
 
@@ -315,6 +328,17 @@ def _convert_options_to_unicode(options):
             setattr(options, key, value.decode(encoding))
 
 
+def _validate_platform(parser, platform):
+    """Validate the platform.
+    Exit with an error if platform is not supported by quickstart or is
+    missing files.
+    """
+    try:
+        platform_support.validate_platform(platform)
+    except ValueError as err:
+        return parser.error(bytes(err))
+
+
 def setup():
     """Set up the application options and logger.
 
@@ -329,7 +353,12 @@ def setup():
         - env_name: the name of the Juju environment to use;
         - env_type: the provider type of the selected Juju environment;
         - interactive: whether to start the interactive session;
-        - open_browser: whether the GUI browser must be opened.
+        - open_browser: whether the GUI browser must be opened;
+        - platform: The host platform;
+        - upload_tools: whether to upload local version of tools;
+        - upload_series: the comma-separated series list for which tools will
+          be uploaded, or None if not set;
+        - constraints: the environment constrains or None if not set.
 
     The following attributes will also be included in the namespace if a bundle
     deployment is requested:
@@ -341,9 +370,14 @@ def setup():
 
     Exit with an error if the provided arguments are not valid.
     """
+
+    # Determine the host platform.  This needs to be done early as it affects
+    # the options we present.
+    platform = platform_support.get_platform()
+
     default_env_name = envs.get_default_env_name()
     default_distro_only, distro_only_help, ppa_help = _get_packaging_info(
-        packaging.JUJU_SOURCE)
+        packaging.JUJU_SOURCE, platform)
     # Define the help message for the --environment option.
     env_help = 'The name of the Juju environment to use'
     if default_env_name is not None:
@@ -402,6 +436,7 @@ def setup():
     parser.add_argument(
         '--no-browser', action='store_false', dest='open_browser',
         help='Avoid opening the browser to the GUI at the end of the\nprocess')
+
     parser.add_argument(
         '--distro-only', action='store_true', dest='distro_only',
         default=default_distro_only, help=distro_only_help)
@@ -419,8 +454,28 @@ def setup():
     parser.add_argument(
         '--description', action=_DescriptionAction, default=argparse.SUPPRESS,
         nargs=0, help="Show program's description and exit")
+    # These options are passed to juju bootstrap.
+    parser.add_argument(
+        '--upload-tools', action='store_true',
+        help='upload local version of tools before bootstrapping')
+    parser.add_argument(
+        '--upload-series',
+        help='upload tools for supplied comma-separated series list')
+    parser.add_argument(
+        '--constraints',
+        help='If constraints are specified, they will apply to the machine\n'
+             'provisioned for the Juju state server. \n'
+             'They will also be set as default constraints on the\n'
+             'environment for all future machines, exactly as if the\n'
+             'constraints were set with "juju set-constraints".\n')
     # Parse the provided arguments.
     options = parser.parse_args()
+
+    _validate_platform(parser, platform)
+
+    # Add in the platform for convenience.
+    options.platform = platform
+
     # Convert the provided string arguments to unicode.
     _convert_options_to_unicode(options)
     # Validate and process the provided arguments.
@@ -441,30 +496,37 @@ def run(options):
         print('contents loaded for bundle {} (services: {})'.format(
             options.bundle_name, len(options.bundle_services)))
 
-    logging.debug('ensuring juju and lxc are installed')
-    juju_version = app.ensure_dependencies(options.distro_only)
+    juju_command = platform_support.get_juju_command(options.platform)
+
+    logging.debug('ensuring juju and dependencies are installed')
+    juju_version = app.ensure_dependencies(
+        options.distro_only, options.platform, juju_command)
 
     logging.debug('ensuring SSH keys are available')
     app.ensure_ssh_keys()
 
     print('bootstrapping the {} environment (type: {})'.format(
         options.env_name, options.env_type))
-    is_local = options.env_type == 'local'
     requires_sudo = False
-    if is_local:
+    if options.env_type == 'local':
         # If this is a local environment, notify the user that "sudo" will be
         # required to bootstrap the application, even in newer Juju versions
         # where "sudo" is invoked by juju-core itself.
         print('sudo privileges will be required to bootstrap the environment')
         # If the Juju version is less than 1.17.2 then use sudo for local envs.
         requires_sudo = juju_version < (1, 17, 2)
-    already_bootstrapped, bsn_series = app.bootstrap(
-        options.env_name, requires_sudo=requires_sudo, debug=options.debug)
+
+    already_bootstrapped, bootstrap_node_series = app.bootstrap(
+        options.env_name, juju_command,
+        requires_sudo=requires_sudo, debug=options.debug,
+        upload_tools=options.upload_tools,
+        upload_series=options.upload_series,
+        constraints=options.constraints)
 
     # Retrieve the admin-secret for the current environment.
     try:
-        admin_secret = app.get_admin_secret(
-            options.env_name, settings.JUJU_HOME)
+        admin_secret = app.get_value_from_jenv(
+            options.env_name, settings.JUJU_HOME, 'admin-secret')
     except ValueError as err:
         admin_secret = options.admin_secret
         if admin_secret is None:
@@ -474,19 +536,19 @@ def run(options):
             raise app.ProgramExit(msg)
 
     print('retrieving the Juju API address')
-    api_url = app.get_api_url(options.env_name)
+    api_url = app.get_api_url(options.env_name, juju_command)
     print('connecting to {}'.format(api_url))
     env = app.connect(api_url, admin_secret)
 
-    # It is not possible to deploy on the bootstrap node if we are using the
-    # local provider, or if the bootstrap node series is not compatible with
-    # the Juju GUI charm.
-    machine = '0'
-    if is_local or (bsn_series != settings.JUJU_GUI_PREFERRED_SERIES):
-        machine = None
+    # Inspect the environment and deploy the charm if required.
+    charm_url, machine, service_data, unit_data = app.check_environment(
+        env, settings.JUJU_GUI_SERVICE_NAME, options.charm_url,
+        options.env_type, bootstrap_node_series, already_bootstrapped)
     unit_name = app.deploy_gui(
-        env, settings.JUJU_GUI_SERVICE_NAME, machine,
-        charm_url=options.charm_url, check_preexisting=already_bootstrapped)
+        env, settings.JUJU_GUI_SERVICE_NAME, charm_url, machine,
+        service_data, unit_data)
+
+    # Observe the deployment progress.
     address = app.watch(env, unit_name)
     env.close()
     url = 'https://{}'.format(address)
