@@ -344,7 +344,14 @@ def create_local_env_data(env_type_db, name, is_default=False):
     either optional or suitable for automatic generation of their values. For
     this reason local environments can be safely created given just their name.
     """
-    env_data = {'type': 'local', 'name': name, 'is-default': is_default}
+    env_data = {
+        'type': 'local',
+        'name': name,
+        'is-default': is_default,
+        # Workaround for bug 1306537: force tools for the most recent Juju GUI
+        # supported series to be uploaded.
+        'default-series': settings.JUJU_GUI_SUPPORTED_SERIES[-1],
+    }
     env_metadata = get_env_metadata(env_type_db, env_data)
     # Retrieve a list of missing required fields.
     missing_fields = [
@@ -402,7 +409,6 @@ def get_env_type_db():
     default_series_field = fields.ChoiceField(
         'default-series', choices=settings.JUJU_DEFAULT_SERIES,
         label='default series', required=False,
-        default=settings.JUJU_GUI_PREFERRED_SERIES,
         help='the default Ubuntu series to use for the bootstrap node')
     is_default_field = fields.BoolField(
         'is-default', label='default', allow_mixed=False, required=True,
@@ -413,11 +419,17 @@ def get_env_type_db():
         'eu-west-1', 'sa-east-1',
         'us-east-1', 'us-west-1', 'us-west-2',
     )
-    hp_regions = (
-        'az-1.region-a.geo-1', 'az-2.region-a.geo-1', 'az-3.region-a.geo-1')
+    hp_regions = ('region-a.geo-1', 'region-b.geo-1')
     azure_locations = (
         'East US', 'West US', 'West Europe', 'North Europe',
         'Southeast Asia', 'East Asia')
+    # The first URL is used as the default one.
+    joyent_sdc_urls = (
+        'https://us-west-1.api.joyentcloud.com',
+        'https://us-east-1.api.joyentcloud.com',
+        'https://us-sw-1.api.joyentcloud.com',
+        'https://us-ams-1.api.joyentcloud.com',
+    )
     # Define the env_type_db dictionary: this is done inside this function in
     # order to avoid instantiating fields at import time.
     # This is an ordered dict so that views can expose options to create new
@@ -470,7 +482,7 @@ def get_env_type_db():
                 help='The secret key to use to authenticate to AWS. '
                      'See the access key help above.'),
             fields.AutoGeneratedStringField(
-                'control-bucket', label='control bucket', required=True,
+                'control-bucket', label='control bucket', required=False,
                 help='the globally unique S3 bucket name'),
             fields.ChoiceField(
                 'region', choices=ec2_regions, default='us-east-1',
@@ -499,9 +511,10 @@ def get_env_type_db():
                 help='Specifies whether the use of a floating IP address is '
                      'required to give the nodes a public IP address. '
                      'Some installations assign public IP addresses by '
-                     'default without requiring a floating IP address.'),
+                     'default without requiring a floating IP address. '
+                     'For HP Public Cloud, floating IP must be activated.'),
             fields.AutoGeneratedStringField(
-                'control-bucket', label='control bucket', required=True,
+                'control-bucket', label='control bucket', required=False,
                 help='the globally unique swift bucket name'),
             fields.SuggestionsStringField(
                 'auth-url', label='authentication URL', required=True,
@@ -513,7 +526,7 @@ def get_env_type_db():
                 'tenant-name', label='tenant name', required=True,
                 help='The OpenStack tenant name. For HP Public Cloud, this is '
                      'listed as the project name on the '
-                     'https://account.hpcloud.com/projects page.'),
+                     'https://horizon.hpcloud.com/landing/ page.'),
             fields.SuggestionsStringField(
                 'region', label='region', required=True,
                 help='The OpenStack region to use. '
@@ -525,15 +538,17 @@ def get_env_type_db():
                 help='The way Juju authenticates to OpenStack. The userpass '
                      'authentication requires you to fill in your user name '
                      'and password. The keypair mode requires access key and '
-                     'secret key to be properly set up. For HP Public Cloud '
-                     'these information can be retrieved on the '
-                     'https://account.hpcloud.com/account/api_keys page.'),
+                     'secret key to be properly set up. For HP Public Cloud, '
+                     'select userpass mode and type your HP Cloud login '
+                     'user name and password below.'),
             fields.StringField(
                 'username', label='user name', required=False,
-                help='the user name to use for the userpass authentication'),
+                help='the user name to use for the userpass authentication '
+                     '(or the HP Cloud login user name)'),
             fields.PasswordField(
                 'password', label='password', required=False,
-                help='the user name to use for the userpass authentication'),
+                help='the user name to use for the userpass authentication '
+                     '(or the HP Cloud login password)'),
             fields.StringField(
                 'access-key', label='access key', required=False,
                 help='the access key to use for the keypair authentication'),
@@ -566,7 +581,7 @@ def get_env_type_db():
                 label='management subscription ID',
                 help='this information can be retrieved from '
                      'https://manage.windowsazure.com (Settings)'),
-            fields.StringField(
+            fields.FilePathField(
                 'management-certificate-path', required=True,
                 label='management certificate path',
                 help='the path to the pem file associated to the certificate '
@@ -584,6 +599,60 @@ def get_env_type_db():
             is_default_field,
         ),
     }
+    env_type_db['joyent'] = {
+        'label': 'Joyent',
+        'description': (
+            'The joyent provider enables you to run Juju on the Joyent cloud. '
+            'This process requires you to have a Joyent account. If you have '
+            'not signed up for one yet, it can obtained at '
+            'http://www.joyent.com/. See '
+            'https://juju.ubuntu.com/docs/config-joyent.html for more details '
+            'on the joyent provider configuration.'
+        ),
+        'fields': (
+            provider_field,
+            name_field,
+            admin_secret_field,
+            default_series_field,
+            fields.StringField(
+                'sdc-user', required=True,
+                label='SDC user name',
+                help='The Smart Data Center account name. '
+                     'This is usually your Joyent user name.'),
+            fields.StringField(
+                'sdc-key-id', required=True,
+                label='SDC key id',
+                help='the SSH public key fingerprint used to connect to the '
+                     'Smart Data Center'),
+            fields.ChoiceField(
+                'sdc-url', choices=joyent_sdc_urls, label='SDC URL',
+                required=False, default=joyent_sdc_urls[0],
+                help='the region URL to use for SDC'),
+            fields.StringField(
+                'manta-user', required=True,
+                label='manta user name',
+                help='The Manta Storage Service account name. '
+                     'This is usually your Joyent user name.'),
+            fields.StringField(
+                'manta-key-id', required=True,
+                label='manta key id',
+                help='The SSH public key fingerprint used to connect to the '
+                     'Manta Storage Service. This is usually the same SSH '
+                     'fingerprint provided as SDC key id.'),
+            fields.StringField(
+                'manta-url', label='manta URL', required=False,
+                default='https://us-east.manta.joyent.com',
+                help='the region URL to use for manta'),
+            fields.FilePathField(
+                'private-key-path', label='private key path', required=False,
+                default='~/.ssh/id_rsa',
+                help='the path to the SSH private key'),
+            fields.StringField(
+                'algorithm', label='SSH algorithm', required=False,
+                default='rsa-sha256'),
+            is_default_field,
+        ),
+    }
     env_type_db['local'] = {
         'label': 'local (LXC)',
         'description': (
@@ -614,21 +683,36 @@ def get_env_type_db():
             fields.StringField(
                 'network-bridge', label='network bridge', required=False,
                 default='lxcbr0', help='the LXC bridge interface to use'),
+            fields.BoolField(
+                'lxc-clone', label='use lxc-clone', required=False,
+                help='Use lxc-clone to create the containers used as '
+                     'machines. LXC clone is enabled by default for Trusty '
+                     'and above, and disabled for earlier Ubuntu releases.'),
+            fields.BoolField(
+                'lxc-clone-aufs', label='use lxc-clone-aufs', required=False,
+                help='Use aufs as a backing-store for the LXC clones. Note '
+                     "that there are some situations where aufs doesn't "
+                     'entirely behave as intuitively as one might expect. '
+                     'If not specified, aufs is not used.'),
             is_default_field,
         ),
     }
     return env_type_db
 
 
-def get_supported_env_types(env_type_db):
+def get_supported_env_types(env_type_db, filter_function=None):
     """Return a list of supported (provider type, label) tuples.
 
     Each tuple represents an environment type supported by Quickstart.
+    It is possible to filter results by providing a filter_function callable
+    which receives the environment type and metadata.
     """
+    if filter_function is None:
+        filter_function = lambda env_type, metadata: True
     return [
         (env_type, metadata['label'])
         for env_type, metadata in env_type_db.items()
-        if env_type != '__fallback__'
+        if (env_type != '__fallback__') and filter_function(env_type, metadata)
     ]
 
 
