@@ -36,6 +36,18 @@ from quickstart.tests import helpers
 from quickstart.tests.cli import helpers as cli_helpers
 
 
+def local_envs_supported(value):
+    """Simulate local environments support in the current platform.
+
+    The value argument is a boolean representing whether or not local
+    environments are supported.
+    Return a context manager that can be used when calling views.
+    """
+    return mock.patch(
+        'quickstart.cli.views.platform_support.supports_local',
+        mock.Mock(return_value=value))
+
+
 class TestShow(unittest.TestCase):
 
     @contextmanager
@@ -125,6 +137,8 @@ class EnvViewTestsMixin(cli_helpers.CliAppTestsMixin):
 class TestEnvIndex(EnvViewTestsMixin, unittest.TestCase):
 
     base_status = ' \N{UPWARDS ARROW LEFTWARDS OF DOWNWARDS ARROW} navigate '
+    create_local_caption = (
+        '\N{BULLET} automatically create and bootstrap a local environment')
 
     def test_view_default_return_value_on_exit(self):
         # The view configures the app so that the return value on user exit is
@@ -157,7 +171,9 @@ class TestEnvIndex(EnvViewTestsMixin, unittest.TestCase):
         # The view displays a list of the environments in env_db, and buttons
         # to create new environments.
         env_db = helpers.make_env_db()
-        views.env_index(self.app, self.env_type_db, env_db, self.save_callable)
+        with local_envs_supported(True):
+            views.env_index(
+                self.app, self.env_type_db, env_db, self.save_callable)
         buttons = self.get_widgets_in_contents(
             filter_function=self.is_a(ui.MenuButton))
         # A button is created for each existing environment (see details) and
@@ -166,6 +182,22 @@ class TestEnvIndex(EnvViewTestsMixin, unittest.TestCase):
         expected_buttons_number = len(env_db['environments']) + len(env_types)
         self.assertEqual(expected_buttons_number, len(buttons))
 
+    def test_new_local_environment_disabled(self):
+        # The option to create a new local environment is not present if they
+        # are not supported in the current platform.
+        env_db = helpers.make_env_db()
+        with local_envs_supported(False):
+            views.env_index(
+                self.app, self.env_type_db, env_db, self.save_callable)
+        buttons = self.get_widgets_in_contents(
+            filter_function=self.is_a(ui.MenuButton))
+        captions = map(cli_helpers.get_button_caption, buttons)
+        create_local_captions = [
+            caption for caption in captions
+            if caption.startswith('\N{BULLET} new local')
+        ]
+        self.assertEqual([], create_local_captions)
+
     @mock.patch('quickstart.cli.views.env_detail')
     def test_environment_clicked(self, mock_env_detail):
         # The environment detail view is called when clicking an environment.
@@ -196,7 +228,9 @@ class TestEnvIndex(EnvViewTestsMixin, unittest.TestCase):
         # The environment edit view is called when clicking to create a new
         # environment.
         env_db = helpers.make_env_db()
-        views.env_index(self.app, self.env_type_db, env_db, self.save_callable)
+        with local_envs_supported(True):
+            views.env_index(
+                self.app, self.env_type_db, env_db, self.save_callable)
         buttons = self.get_widgets_in_contents(
             filter_function=self.is_a(ui.MenuButton))
         env_types = envs.get_supported_env_types(self.env_type_db)
@@ -212,7 +246,7 @@ class TestEnvIndex(EnvViewTestsMixin, unittest.TestCase):
             mock_env_edit.assert_called_once_with(
                 self.app, self.env_type_db, env_db, self.save_callable,
                 {'type': env_type,
-                 'default-series': settings.JUJU_GUI_PREFERRED_SERIES})
+                 'default-series': settings.JUJU_GUI_SUPPORTED_SERIES[-1]})
             # Reset the mock so that it does not include any calls on the next
             # loop cycle.
             mock_env_edit.reset_mock()
@@ -223,14 +257,16 @@ class TestEnvIndex(EnvViewTestsMixin, unittest.TestCase):
         # If that option is clicked, the view quits the application returning
         # the newly created env_data.
         env_db = envs.create_empty_env_db()
-        views.env_index(self.app, self.env_type_db, env_db, self.save_callable)
+        with local_envs_supported(True):
+            views.env_index(
+                self.app, self.env_type_db, env_db, self.save_callable)
         buttons = self.get_widgets_in_contents(
             filter_function=self.is_a(ui.MenuButton))
         # The "create and bootstrap" button is the first one in the contents.
         create_button = buttons[0]
         self.assertEqual(
-            '\N{BULLET} automatically create and bootstrap a local '
-            'environment', cli_helpers.get_button_caption(create_button))
+            self.create_local_caption,
+            cli_helpers.get_button_caption(create_button))
         # An AppExit is raised clicking the button.
         with self.assertRaises(ui.AppExit) as context_manager:
             cli_helpers.emit(create_button)
@@ -239,6 +275,20 @@ class TestEnvIndex(EnvViewTestsMixin, unittest.TestCase):
         self.assertIn('local', new_env_db['environments'])
         self.assertEqual(envs.get_env_data(new_env_db, 'local'), env_data)
 
+    def test_create_and_bootstrap_local_environment_missing(self):
+        # The option to automatically create and bootstrap a new local
+        # environment is not displayed if the current platform does not support
+        # local environments.
+        env_db = envs.create_empty_env_db()
+        with local_envs_supported(False):
+            views.env_index(
+                self.app, self.env_type_db, env_db, self.save_callable)
+        buttons = self.get_widgets_in_contents(
+            filter_function=self.is_a(ui.MenuButton))
+        # No "create and bootstrap" buttons are present.
+        captions = map(cli_helpers.get_button_caption, buttons)
+        self.assertNotIn(self.create_local_caption, captions)
+
     def test_selected_environment(self):
         # The default environment is already selected in the list.
         env_db = helpers.make_env_db(default='lxc')
@@ -589,6 +639,8 @@ class TestEnvEdit(EnvViewTestsMixin, unittest.TestCase):
             ('storage port: ', '8888'),
             ('shared storage port: ', ''),
             ('network bridge: ', ''),
+            ('use lxc-clone', forms.MIXED),
+            ('use lxc-clone-aufs', forms.MIXED),
             ('default', True),
         ]
         self.assertEqual(expected_form_contents, self.get_form_contents())
