@@ -29,6 +29,7 @@ import unittest
 import mock
 import yaml
 
+from quickstart import settings
 from quickstart.models import (
     envs,
     fields,
@@ -619,8 +620,13 @@ class TestCreateLocalEnvData(unittest.TestCase):
         # value should be the admin-secret.
         admin_secret = env_data.pop('admin-secret', '')
         self.assertNotEqual(0, len(admin_secret))
-        expected = {'type': 'local', 'name': 'my-lxc', 'is-default': False}
-        self.assertEqual(expected, env_data)
+        expected_env_data = {
+            'type': 'local',
+            'name': 'my-lxc',
+            'is-default': False,
+            'default-series': settings.JUJU_GUI_SUPPORTED_SERIES[-1],
+        }
+        self.assertEqual(expected_env_data, env_data)
 
     def test_default(self):
         # The resulting env_data is correctly structured for default envs.
@@ -629,8 +635,13 @@ class TestCreateLocalEnvData(unittest.TestCase):
         # See the comment about auto-generated fields in the test method above.
         admin_secret = env_data.pop('admin-secret', '')
         self.assertNotEqual(0, len(admin_secret))
-        expected = {'type': 'local', 'name': 'my-default', 'is-default': True}
-        self.assertEqual(expected, env_data)
+        expected_env_data = {
+            'type': 'local',
+            'name': 'my-default',
+            'is-default': True,
+            'default-series': settings.JUJU_GUI_SUPPORTED_SERIES[-1],
+        }
+        self.assertEqual(expected_env_data, env_data)
 
 
 class TestRemoveEnv(
@@ -731,6 +742,7 @@ class TestGetEnvTypeDb(unittest.TestCase):
         expected = [
             'type', 'name', 'admin-secret', 'default-series', 'root-dir',
             'storage-port', 'shared-storage-port', 'network-bridge',
+            'lxc-clone', 'lxc-clone-aufs',
             'is-default']
         expected_required = ['type', 'name', 'is-default']
         self.assert_fields(expected, env_metadata)
@@ -745,7 +757,7 @@ class TestGetEnvTypeDb(unittest.TestCase):
             'secret-key', 'control-bucket', 'region', 'is-default']
         expected_required = [
             'type', 'name', 'access-key', 'secret-key',
-            'control-bucket', 'is-default']
+            'is-default']
         self.assert_fields(expected, env_metadata)
         self.assert_required_fields(expected_required, env_metadata)
 
@@ -760,8 +772,7 @@ class TestGetEnvTypeDb(unittest.TestCase):
             'secret-key', 'is-default']
         expected_required = [
             'type', 'name', 'use-floating-ip',
-            'control-bucket', 'auth-url', 'tenant-name', 'region',
-            'is-default']
+            'auth-url', 'tenant-name', 'region', 'is-default']
         self.assert_fields(expected, env_metadata)
         self.assert_required_fields(expected_required, env_metadata)
 
@@ -780,19 +791,49 @@ class TestGetEnvTypeDb(unittest.TestCase):
         self.assert_fields(expected, env_metadata)
         self.assert_required_fields(expected_required, env_metadata)
 
+    def test_joyent_environment(self):
+        # The joyent environment metadata includes the expected fields.
+        self.assertIn('joyent', self.env_type_db)
+        env_metadata = self.env_type_db['joyent']
+        expected = [
+            'type', 'name', 'admin-secret', 'default-series',
+            'sdc-user', 'sdc-key-id', 'sdc-url',
+            'manta-user', 'manta-key-id', 'manta-url',
+            'private-key-path', 'algorithm', 'is-default']
+        expected_required = [
+            'type', 'name', 'sdc-user', 'sdc-key-id', 'manta-user',
+            'manta-key-id', 'is-default']
+        self.assert_fields(expected, env_metadata)
+        self.assert_required_fields(expected_required, env_metadata)
+
 
 class TestGetSupportedEnvTypes(unittest.TestCase):
 
+    def setUp(self):
+        # Store the environments database.
+        self.env_type_db = envs.get_env_type_db()
+
     def test_env_types(self):
         # All the supported env_types but the fallback one are returned.
-        env_type_db = envs.get_env_type_db()
         expected_env_types = [
             ('ec2', 'Amazon EC2'),
             ('openstack', 'OpenStack (or HP Public Cloud)'),
             ('azure', 'Windows Azure'),
+            ('joyent', 'Joyent'),
             ('local', 'local (LXC)'),
         ]
-        obtained_env_types = envs.get_supported_env_types(env_type_db)
+        obtained_env_types = envs.get_supported_env_types(self.env_type_db)
+        self.assertEqual(expected_env_types, obtained_env_types)
+
+    def test_filter_function(self):
+        # Results can be filtered by providing a filter function.
+        expected_env_types = [
+            ('ec2', 'Amazon EC2'),
+            ('joyent', 'Joyent'),
+        ]
+        func = lambda env_type, metadata: env_type in ('ec2', 'joyent')
+        obtained_env_types = envs.get_supported_env_types(
+            self.env_type_db, filter_function=func)
         self.assertEqual(expected_env_types, obtained_env_types)
 
 
@@ -844,6 +885,8 @@ class TestMapFieldsToEnvData(unittest.TestCase):
             ('storage-port', 4242),
             ('shared-storage-port', 4747),
             ('network-bridge', 'lxcbr1'),
+            ('lxc-clone', False),
+            ('lxc-clone-aufs', False),
             ('is-default', True),
         ]
 
@@ -864,6 +907,8 @@ class TestMapFieldsToEnvData(unittest.TestCase):
             ('storage-port', None),
             ('shared-storage-port', None),
             ('network-bridge', None),
+            ('lxc-clone', None),
+            ('lxc-clone-aufs', None),
             ('is-default', None),
         ]
         env_data = {'type': 'local', 'name': 'lxc'}
