@@ -19,8 +19,11 @@
 from __future__ import unicode_literals
 
 from contextlib import contextmanager
+import os
 import unittest
 
+import mock
+
 from quickstart.models import fields
 from quickstart.tests import helpers
 
@@ -457,6 +460,37 @@ class TestChoiceField(TestStringField):
             field.validate(None)
 
 
+class TestFilePathField(TestStringField):
+
+    field_class = fields.FilePathField
+
+    def test_validation_success(self):
+        # The validation succeeds if the value is set and the path exists.
+        field = self.field_class('path')
+        with self.assert_not_raises(ValueError):
+            field.validate(unicode(__file__))
+
+    def test_validation_expanduser(self):
+        # The ~ construct is properly expanded.
+        field = self.field_class('path')
+        basename, filename = os.path.split(unicode(__file__))
+        with self.assert_not_raises(ValueError):
+            with mock.patch('os.environ', {'HOME': basename}):
+                field.validate('~/{}'.format(filename))
+
+    def test_validation_error_file_not_found(self):
+        # A ValueError is raised if the path does not exist.
+        field = self.field_class('path')
+        with self.assert_value_error('file not found in the specified path'):
+            field.validate('__no-such-file__')
+
+    def test_validation_error_directory(self):
+        # A ValueError is raised if the path is a directory.
+        field = self.field_class('path')
+        with self.assert_value_error('file not found in the specified path'):
+            field.validate('/tmp')
+
+
 class TestSuggestionsStringField(TestStringField):
 
     field_class = fields.SuggestionsStringField
