@@ -63,24 +63,34 @@ class TestGetPackagingInfo(unittest.TestCase):
     distro_only_disable = '(enabled by default, use --ppa to disable)'
     ppa_disable = '(enabled by default, use --distro-only to disable)'
 
-    def test_ppa_source(self):
+    def test_ppa_source_apt(self):
         # The returned distro_only flag is set to False and the help texts are
         # formatted accordingly when the passed Juju source is "ppa".
         distro_only, distro_only_help, ppa_help = manage._get_packaging_info(
-            'ppa')
+            'ppa', platform=settings.LINUX_APT)
         self.assertFalse(distro_only)
         self.assertNotIn(self.distro_only_disable, distro_only_help)
         self.assertIn(self.ppa_disable, ppa_help)
 
-    def test_distro_source(self):
+    def test_distro_source_apt(self):
         # The returned distro_only flag is set to True and the help texts are
         # formatted accordingly when the passed Juju source is "distro".
         distro_only, distro_only_help, ppa_help = manage._get_packaging_info(
-            'distro')
+            'distro', platform=settings.LINUX_APT)
         self.assertTrue(distro_only)
         self.assertIn(self.distro_only_disable, distro_only_help)
         self.assertNotIn(self.ppa_disable, ppa_help)
 
+    def test_not_apt(self):
+        # If the platform is not LINUX_APT then distro_only should be False
+        # and the help is suppressed.
+        for platform in (settings.LINUX_RPM, settings.WINDOWS, settings.OSX):
+            distro_only, dist_only_help, ppa_help = manage._get_packaging_info(
+                'dontcare', platform=platform)
+            self.assertTrue(distro_only)
+            self.assertEqual(argparse.SUPPRESS, dist_only_help)
+            self.assertEqual(argparse.SUPPRESS, ppa_help)
+
 
 class TestValidateBundle(
         helpers.BundleFileTestsMixin, helpers.UrlReadTestsMixin,
@@ -444,17 +454,43 @@ class TestRetrieveEnvData(unittest.TestCase):
             'the storage port field requires an integer value')
 
 
+class TestValidatePlatform(unittest.TestCase):
+
+    def setUp(self):
+        # Set up a parser, the environments metadata and a testing env_db.
+        self.parser = mock.Mock()
+        self.env_type_db = envs.get_env_type_db()
+        self.env_db = helpers.make_env_db()
+
+    def test_platform_validation_fails(self):
+        # If the platform validation fails a parser error is given.
+        path = 'quickstart.manage.platform_support.validate_platform'
+        with mock.patch(path, side_effect=ValueError('Bad platform, yo')):
+            manage._validate_platform(self.parser, settings.LINUX_RPM)
+        self.parser.error.assert_called_once_with(
+            'Bad platform, yo')
+
+    def test_platform_validation_passes(self):
+        # If the platform validation passes it returns None.
+        path = 'quickstart.platform_support.validate_platform'
+        with mock.patch(path, side_effect=None):
+            result = manage._validate_platform(self.parser, settings.LINUX_RPM)
+        self.assertIsNone(result)
+
+
 class TestSetupEnv(helpers.EnvFileTestsMixin, unittest.TestCase):
 
     def setUp(self):
         self.parser = mock.Mock()
 
-    def make_options(self, env_file, env_name=None, interactive=False):
+    def make_options(self, env_file, env_name=None, interactive=False,
+                     platform=settings.LINUX_APT):
         """Return a mock options object which includes the passed arguments."""
         return mock.Mock(
             env_file=env_file,
             env_name=env_name,
             interactive=interactive,
+            platform=platform,
         )
 
     def patch_interactive_mode(self, return_value):
@@ -503,7 +539,7 @@ class TestSetupEnv(helpers.EnvFileTestsMixin, unittest.TestCase):
         message = self.parser.error.call_args[0][0]
         self.assertIn('unable to find an environment name to use', message)
 
-    def test_local_provider(self):
+    def test_local_provider_linux(self):
         # Local environments are correctly handled.
         contents = yaml.safe_dump({
             'environments': {
@@ -521,6 +557,24 @@ class TestSetupEnv(helpers.EnvFileTestsMixin, unittest.TestCase):
         self.assertIsNone(options.default_series)
         self.assertFalse(options.interactive)
 
+    def test_local_provider_not_linux(self):
+        # Local environments are correctly handled.
+        contents = yaml.safe_dump({
+            'environments': {
+                'lxc': {'admin-secret': 'Secret!', 'type': 'local'},
+            },
+        })
+        env_file = self.make_env_file(contents)
+        options = self.make_options(
+            env_file, env_name='lxc', interactive=False,
+            platform=settings.OSX)
+        manage._setup_env(options, self.parser)
+        self.assertTrue(self.parser.error.called)
+        message = self.parser.error.call_args[0][0]
+        self.assertEqual(
+            'this host platform does not support local environments',
+            message)
+
     def test_interactive_mode(self):
         # The interactive mode is started properly if the corresponding option
         # flag is set.
@@ -606,9 +660,10 @@ class TestSetup(unittest.TestCase):
         with mock.patch('sys.argv', ['juju-quickstart'] + args):
             with mock.patch('sys.exit') as mock_exit:
                 with self.patch_get_default_env_name(env_name):
-                    manage.setup()
+                    options = manage.setup()
         if exit_called:
             mock_exit.assert_called_once_with(0)
+        return options
 
     def test_help(self):
         # The program help message is properly formatted.
@@ -681,7 +736,12 @@ class TestSetup(unittest.TestCase):
         # Logging is properly set up at the debug level.
         logger = logging.getLogger()
         self.call_setup(['--debug'], 'ec2', exit_called=False)
-        self.assertEqual(logging.DEBUG, logger.level)
+        self.assertTrue(logging.DEBUG, logger.level)
+
+    def test_upload_tools(self):
+        # Upload tools is properly set up.
+        options = self.call_setup(['--upload-tools'], exit_called=False)
+        self.assertTrue(options.upload_tools)
 
 
 @mock.patch('webbrowser.open')
@@ -689,10 +749,12 @@ class TestSetup(unittest.TestCase):
 @mock.patch('__builtin__.print', mock.Mock())
 class TestRun(unittest.TestCase):
 
+    juju_command = '/sbin/juju'
+
     def make_options(self, **kwargs):
         """Set up the options to be passed to the run function."""
         options = {
-            'admin_secret': 'Secret!',
+            'admin_secret': 'Secretz!',
             'bundle': None,
             'bundle_id': None,
             'charm_url': None,
@@ -706,40 +768,60 @@ class TestRun(unittest.TestCase):
         return mock.Mock(**options)
 
     @staticmethod
-    def mock_get_admin_secret_success(name, home):
-        return 'jenv secret'
+    def mock_get_value_from_jenv_success(name, home, key):
+        return 'from jenv'
 
     @staticmethod
-    def mock_get_admin_secret_error(name, home):
+    def mock_get_value_from_jenv_error(name, home, key):
         fn = '{}.jenv'.format(name)
         path = os.path.join(home, 'environments', fn)
-        msg = 'admin-secret not found in {}'.format(path)
+        msg = '{} not found in {}'.format(key, path)
         raise ValueError(msg.encode('utf-8'))
 
     def test_no_bundle(self, mock_app, mock_open):
         # The application runs correctly if no bundle is provided.
         mock_app.ensure_dependencies.return_value = (1, 18, 0)
-        mock_app.bootstrap.return_value = (True, 'precise')
-        mock_app.get_admin_secret = self.mock_get_admin_secret_error
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
+        mock_app.bootstrap.return_value = (True, 'trusty')
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        service_data = {'Name': 'juju-gui'}
+        unit_data = {'Name': 'juju-gui/0'}
+        mock_app.check_environment.return_value = (
+            'cs:trusty/juju-gui-42', '0', service_data, unit_data)
+        mock_app.get_value_from_jenv = self.mock_get_value_from_jenv_error
+        # Make mock_app.watch return the Juju GUI unit address.
         mock_app.watch.return_value = '1.2.3.4'
+        # Make mock_app.create_auth_token return a fake authentication token.
         mock_app.create_auth_token.return_value = 'AUTHTOKEN'
         options = self.make_options()
-        manage.run(options)
+        with mock.patch('quickstart.manage.platform_support.get_juju_command',
+                        side_effect=[self.juju_command]):
+            manage.run(options)
         mock_app.ensure_dependencies.assert_called()
         mock_app.ensure_ssh_keys.assert_called()
         mock_app.bootstrap.assert_called_once_with(
-            options.env_name, requires_sudo=False, debug=options.debug)
-        mock_app.get_api_url.assert_called_once_with(options.env_name)
+            options.env_name, self.juju_command, requires_sudo=False,
+            debug=options.debug, upload_tools=options.upload_tools,
+            upload_series=options.upload_series,
+            constraints=options.constraints)
+        mock_app.get_api_url.assert_called_once_with(
+            options.env_name, self.juju_command)
         mock_app.connect.assert_has_calls([
-            mock.call(mock_app.get_api_url(), options.admin_secret),
+            mock.call(
+                mock_app.get_api_url(), options.admin_secret),
             mock.call().close(),
             mock.call('wss://1.2.3.4:443/ws', options.admin_secret),
             mock.call().close(),
         ])
+        mock_app.check_environment.assert_called_once_with(
+            mock_app.connect(), settings.JUJU_GUI_SERVICE_NAME,
+            options.charm_url, options.env_type, mock_app.bootstrap()[1],
+            mock_app.bootstrap()[0])
         mock_app.deploy_gui.assert_called_once_with(
-            mock_app.connect(), settings.JUJU_GUI_SERVICE_NAME, '0',
-            charm_url=options.charm_url,
-            check_preexisting=mock_app.bootstrap()[0])
+            mock_app.connect(), settings.JUJU_GUI_SERVICE_NAME,
+            'cs:trusty/juju-gui-42', '0', service_data, unit_data)
         mock_app.watch.assert_called_once_with(
             mock_app.connect(), mock_app.deploy_gui())
         mock_app.create_auth_token.assert_called_once_with(mock_app.connect())
@@ -748,8 +830,16 @@ class TestRun(unittest.TestCase):
         self.assertFalse(mock_app.deploy_bundle.called)
 
     def test_no_token(self, mock_app, mock_open):
+        # The process continues even if the authentication token cannot be
+        # retrieved.
         mock_app.create_auth_token.return_value = None
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
         mock_app.bootstrap.return_value = (True, 'precise')
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        mock_app.check_environment.return_value = (
+            'cs:precise/juju-gui-42', '0', None, None)
         options = self.make_options()
         manage.run(options)
         mock_app.create_auth_token.assert_called_once_with(mock_app.connect())
@@ -761,7 +851,14 @@ class TestRun(unittest.TestCase):
         options = self.make_options(
             bundle='/my/bundle/file.yaml', bundle_yaml='mybundle: contents',
             bundle_name='mybundle', bundle_services=['service1', 'service2'])
-        mock_app.bootstrap.return_value = (True, 'precise')
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
+        mock_app.bootstrap.return_value = (True, 'trusty')
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        mock_app.check_environment.return_value = (
+            'cs:trusty/juju-gui-42', '0', None, None)
+        # Make mock_app.watch return the Juju GUI unit address.
         mock_app.watch.return_value = 'gui.example.com'
         manage.run(options)
         mock_app.deploy_bundle.assert_called_once_with(
@@ -774,12 +871,24 @@ class TestRun(unittest.TestCase):
         options = self.make_options(env_type='local')
         versions = [
             (1, 17, 2), (1, 17, 10), (1, 18, 0), (1, 18, 2), (2, 16, 1)]
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
         mock_app.bootstrap.return_value = (True, 'precise')
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        mock_app.check_environment.return_value = (
+            'cs:precise/juju-gui-42', '0', None, None)
         for version in versions:
             mock_app.ensure_dependencies.return_value = version
-            manage.run(options)
+            with mock.patch(
+                    'quickstart.manage.platform_support.get_juju_command',
+                    side_effect=[self.juju_command]):
+                manage.run(options)
             mock_app.bootstrap.assert_called_once_with(
-                options.env_name, requires_sudo=False, debug=options.debug)
+                options.env_name, self.juju_command, requires_sudo=False,
+                debug=options.debug, upload_tools=options.upload_tools,
+                upload_series=options.upload_series,
+                constraints=options.constraints)
             mock_app.bootstrap.reset_mock()
 
     def test_local_provider_requiring_sudo(self, mock_app, mock_open):
@@ -788,26 +897,56 @@ class TestRun(unittest.TestCase):
         # Sudo privileges are required if the Juju version is < 1.17.2.
         options = self.make_options(env_type='local')
         versions = [(0, 7, 9), (1, 0, 0), (1, 16, 42), (1, 17, 0), (1, 17, 1)]
-        mock_app.bootstrap.return_value = (True, 'precise')
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
+        mock_app.bootstrap.return_value = (True, 'trusty')
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        mock_app.check_environment.return_value = (
+            'cs:trusty/juju-gui-42', '0', None, None)
         for version in versions:
             mock_app.ensure_dependencies.return_value = version
-            manage.run(options)
+            with mock.patch(
+                    'quickstart.manage.platform_support.get_juju_command',
+                    side_effect=[self.juju_command]):
+                manage.run(options)
             mock_app.bootstrap.assert_called_once_with(
-                options.env_name, requires_sudo=True, debug=options.debug)
+                options.env_name, self.juju_command, requires_sudo=True,
+                debug=options.debug, upload_tools=options.upload_tools,
+                upload_series=options.upload_series,
+                constraints=options.constraints)
             mock_app.bootstrap.reset_mock()
 
     def test_no_local_no_sudo(self, mock_app, mock_open):
         # Sudo privileges are never required for non-local environments.
         options = self.make_options(env_type='ec2')
         mock_app.ensure_dependencies.return_value = (1, 14, 0)
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
         mock_app.bootstrap.return_value = (True, 'precise')
-        manage.run(options)
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        mock_app.check_environment.return_value = (
+            'cs:precise/juju-gui-42', '0', None, None)
+        with mock.patch('quickstart.manage.platform_support.get_juju_command',
+                        side_effect=[self.juju_command]):
+            manage.run(options)
         mock_app.bootstrap.assert_called_once_with(
-            options.env_name, requires_sudo=False, debug=options.debug)
+            options.env_name, self.juju_command,
+            requires_sudo=False,
+            debug=options.debug, upload_tools=options.upload_tools,
+            upload_series=options.upload_series,
+            constraints=options.constraints)
 
     def test_no_browser(self, mock_app, mock_open):
         # It is possible to avoid opening the GUI in the browser.
-        mock_app.bootstrap.return_value = (True, 'precise')
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
+        mock_app.bootstrap.return_value = (True, 'trusty')
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        mock_app.check_environment.return_value = (
+            'cs:trusty/juju-gui-42', '0', None, None)
         options = self.make_options(open_browser=False)
         manage.run(options)
         self.assertFalse(mock_open.called)
@@ -815,19 +954,31 @@ class TestRun(unittest.TestCase):
     def test_admin_secret_fetched(self, mock_app, mock_open):
         # If an admin secret is fetched from jenv it is used, even if one is
         # found in environments.yaml, as set in options.admin_secret.
-        mock_app.get_admin_secret = self.mock_get_admin_secret_success
+        mock_app.get_value_from_jenv = self.mock_get_value_from_jenv_success
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
         mock_app.bootstrap.return_value = (True, 'precise')
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        mock_app.check_environment.return_value = (
+            'cs:precise/juju-gui-42', '0', None, None)
         options = self.make_options(admin_secret='secret in environments.yaml')
         manage.run(options)
         mock_app.connect.assert_has_calls([
-            mock.call(mock_app.get_api_url(), 'jenv secret'),
+            mock.call(mock_app.get_api_url(), 'from jenv'),
         ])
 
     def test_admin_secret_from_environments_yaml(self, mock_app, mock_open):
         # If an admin secret is not fetched from jenv, then the one from
         # environments.yaml is used, as found in options.admin_secret.
-        mock_app.get_admin_secret = self.mock_get_admin_secret_error
+        mock_app.get_value_from_jenv = self.mock_get_value_from_jenv_error
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
         mock_app.bootstrap.return_value = (True, 'precise')
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        mock_app.check_environment.return_value = (
+            'cs:precise/juju-gui-42', '0', None, None)
         options = self.make_options(admin_secret='secret in environments.yaml')
         manage.run(options)
         mock_app.connect.assert_has_calls([
@@ -837,8 +988,14 @@ class TestRun(unittest.TestCase):
     def test_no_admin_secret_found(self, mock_app, mock_open):
         # If admin-secret cannot be found anywhere a ProgramExit is called.
         mock_app.ProgramExit = app.ProgramExit
-        mock_app.get_admin_secret = self.mock_get_admin_secret_error
+        mock_app.get_value_from_jenv = self.mock_get_value_from_jenv_error
+        # Make mock_app.bootstrap return the already_bootstrapped flag and the
+        # bootstrap node series.
         mock_app.bootstrap.return_value = (True, 'precise')
+        # Make mock_app.check_environment return the charm URL, the machine
+        # where to deploy the charm, the service and unit data.
+        mock_app.check_environment.return_value = (
+            'cs:precise/juju-gui-42', '0', None, None)
         options = self.make_options(
             admin_secret=None,
             env_name='local',
@@ -846,6 +1003,6 @@ class TestRun(unittest.TestCase):
         with self.assertRaises(app.ProgramExit) as context:
             manage.run(options)
         expected = (
-            u'admin-secret not found in ~/.juju/environments/local.jenv '
-            'or environments.yaml')
+            u'admin-secret not found in {}/environments/local.jenv '
+            'or environments.yaml'.format(settings.JUJU_HOME))
         self.assertEqual(expected, context.exception.message)
