@@ -36,6 +36,7 @@ from quickstart import (
     settings,
     utils,
 )
+from quickstart.models import charms
 from quickstart.tests import helpers
 
 
@@ -133,9 +134,7 @@ class TestCall(unittest.TestCase):
         retcode, output, error = utils.call('ls', 'no-such-file')
         self.assertNotEqual(0, retcode)
         self.assertEqual('', output)
-        self.assertEqual(
-            'ls: cannot access no-such-file: No such file or directory\n',
-            error)
+        self.assertIn('No such file or directory', error)
 
     def test_invalid_command(self):
         # An error code and the error are returned if the subprocess fails to
@@ -158,19 +157,25 @@ class TestCall(unittest.TestCase):
 
 
 @mock.patch('__builtin__.print', mock.Mock())
-class TestCheckGuiCharmUrl(unittest.TestCase):
+class TestParseGuiCharmUrl(unittest.TestCase):
+
+    def test_charm_instance_returned(self):
+        # A charm instance is correctly returned.
+        charm = utils.parse_gui_charm_url('cs:trusty/juju-gui-42')
+        self.assertIsInstance(charm, charms.Charm)
+        self.assertEqual('cs:trusty/juju-gui-42', charm.url())
 
     def test_customized(self):
         # A customized charm URL is properly logged.
         expected = 'using a customized juju-gui charm'
         with helpers.assert_logs([expected], level='warn'):
-            utils.check_gui_charm_url('cs:~juju-gui/precise/juju-gui-28')
+            utils.parse_gui_charm_url('cs:~juju-gui/precise/juju-gui-28')
 
     def test_outdated(self):
         # An outdated charm URL is properly logged.
         expected = 'charm is outdated and may not support bundle deployments'
         with helpers.assert_logs([expected], level='warn'):
-            utils.check_gui_charm_url('cs:precise/juju-gui-1')
+            utils.parse_gui_charm_url('cs:precise/juju-gui-1')
 
     def test_unexpected(self):
         # An unexpected charm URL is properly logged.
@@ -178,12 +183,12 @@ class TestCheckGuiCharmUrl(unittest.TestCase):
             'unexpected URL for the juju-gui charm: the service may not work '
             'as expected')
         with helpers.assert_logs([expected], level='warn'):
-            utils.check_gui_charm_url('cs:precise/another-gui-42')
+            utils.parse_gui_charm_url('cs:precise/another-gui-42')
 
     def test_official(self):
         # No warnings are logged if an up to date charm is passed.
         with mock.patch('logging.warn') as mock_warn:
-            utils.check_gui_charm_url('cs:precise/juju-gui-100')
+            utils.parse_gui_charm_url('cs:precise/juju-gui-100')
         self.assertFalse(mock_warn.called)
 
 
@@ -304,18 +309,20 @@ class TestGetCharmUrl(helpers.UrlReadTestsMixin, unittest.TestCase):
 
     def test_charm_url(self):
         # The Juju GUI charm URL is correctly returned.
-        contents = json.dumps({'charm': {'url': 'cs:precise/juju-gui-42'}})
+        contents = json.dumps({'charm': {'url': 'cs:trusty/juju-gui-42'}})
         with self.patch_urlread(contents=contents) as mock_urlread:
-            charm_url = utils.get_charm_url()
-        self.assertEqual('cs:precise/juju-gui-42', charm_url)
-        mock_urlread.assert_called_once_with(settings.CHARMWORLD_API)
+            charm_url = utils.get_charm_url('trusty')
+        self.assertEqual('cs:trusty/juju-gui-42', charm_url)
+        mock_urlread.assert_called_once_with(
+            'http://manage.jujucharms.com/api/3/charm/trusty/juju-gui')
 
     def test_io_error(self):
         # IOErrors are properly propagated.
         with self.patch_urlread(error=True) as mock_urlread:
             with self.assertRaises(IOError) as context_manager:
-                utils.get_charm_url()
-        mock_urlread.assert_called_once_with(settings.CHARMWORLD_API)
+                utils.get_charm_url('precise')
+        mock_urlread.assert_called_once_with(
+            'http://manage.jujucharms.com/api/3/charm/precise/juju-gui')
         self.assertEqual('bad wolf', bytes(context_manager.exception))
 
     def test_value_error(self):
@@ -323,8 +330,9 @@ class TestGetCharmUrl(helpers.UrlReadTestsMixin, unittest.TestCase):
         contents = json.dumps({'charm': {}})
         with self.patch_urlread(contents=contents) as mock_urlread:
             with self.assertRaises(ValueError) as context_manager:
-                utils.get_charm_url()
-        mock_urlread.assert_called_once_with(settings.CHARMWORLD_API)
+                utils.get_charm_url('trusty')
+        mock_urlread.assert_called_once_with(
+            'http://manage.jujucharms.com/api/3/charm/trusty/juju-gui')
         self.assertEqual(
             'unable to find the charm URL', bytes(context_manager.exception))
 
@@ -751,17 +759,17 @@ class TestGetJujuVersion(
     def test_return_deconstructed_version(self):
         # Should return a deconstructed juju version.
         with self.patch_call(0, '1.17.1-precise-amd64\n', ''):
-            version = utils.get_juju_version()
+            version = utils.get_juju_version('juju')
         self.assertEqual((1, 17, 1), version)
 
     def test_juju_version_error(self):
         # A ValueError is raised if "juju version" exits with an error.
         with self.patch_call(1, 'foo', 'bad wolf'):
             with self.assert_value_error('bad wolf'):
-                utils.get_juju_version()
+                utils.get_juju_version('juju')
 
     def test_invalid_version_string(self):
         # A ValueError is raised if "juju version" outputs an invalid version.
         with self.patch_call(0, '1.17-precise-amd64', ''):
             with self.assert_value_error('invalid version string: 1.17'):
-                utils.get_juju_version()
+                utils.get_juju_version('juju')
