@@ -29,33 +29,33 @@ from quickstart.tests import helpers
 # Define addresses to be used in tests.
 cloud_addresses = [
     {'NetworkName': '',
-     'NetworkScope': 'public',
+     'Scope': 'public',
      'Type': 'hostname',
      'Value': 'eu-west-1.compute.example.com'},
     {'NetworkName': '',
-     'NetworkScope': 'local-cloud',
+     'Scope': 'local-cloud',
      'Type': 'hostname',
      'Value': 'eu-west-1.example.internal'},
     {'NetworkName': '',
-     'NetworkScope': 'public',
+     'Scope': 'public',
      'Type': 'ipv4',
      'Value': '444.222.444.222'},
     {'NetworkName': '',
-     'NetworkScope': 'local-cloud',
+     'Scope': 'local-cloud',
      'Type': 'ipv4',
      'Value': '10.42.47.10'},
     {'NetworkName': '',
-     'NetworkScope': '',
+     'Scope': '',
      'Type': 'ipv6',
      'Value': 'fe80::92b8:d0ff:fe94:8f8c'},
 ]
 container_addresses = [
     {'NetworkName': '',
-     'NetworkScope': '',
+     'Scope': '',
      'Type': 'ipv4',
      'Value': '10.0.3.42'},
     {'NetworkName': '',
-     'NetworkScope': '',
+     'Scope': '',
      'Type': 'ipv6',
      'Value': 'fe80::216:3eff:fefd:787e'},
 ]
@@ -71,11 +71,11 @@ class TestRetrievePublicAddress(unittest.TestCase):
         # None is returned if a cloud machine public address is not available.
         addresses = [
             {'NetworkName': '',
-             'NetworkScope': 'local-cloud',
+             'Scope': 'local-cloud',
              'Type': 'hostname',
              'Value': 'eu-west-1.example.internal'},
             {'NetworkName': '',
-             'NetworkScope': 'local-cloud',
+             'Scope': 'local-cloud',
              'Type': 'ipv4',
              'Value': '10.42.47.10'},
         ]
@@ -85,7 +85,7 @@ class TestRetrievePublicAddress(unittest.TestCase):
         # None is returned if an LXC public address is not available.
         addresses = [{
             'NetworkName': '',
-            'NetworkScope': '',
+            'Scope': '',
             'Type': 'ipv6',
             'Value': 'fe80::216:3eff:fefd:787e',
         }]
@@ -95,11 +95,11 @@ class TestRetrievePublicAddress(unittest.TestCase):
         # None is returned if the public address has no value.
         addresses = [
             {'NetworkName': '',
-             'NetworkScope': 'local-cloud',
+             'Scope': 'local-cloud',
              'Type': 'hostname',
              'Value': 'eu-west-1.example.internal'},
             {'NetworkName': '',
-             'NetworkScope': 'public',
+             'Scope': 'public',
              'Type': 'ipv4',
              'Value': ''},
         ]
@@ -115,15 +115,32 @@ class TestRetrievePublicAddress(unittest.TestCase):
         public_address = watchers.retrieve_public_adddress(container_addresses)
         self.assertEqual('10.0.3.42', public_address)
 
+    def test_old_juju_version(self):
+        # The public address is properly returned when using Juju < 1.20.
+        # Prior to Juju 1.20, the MachineInfo field for the address scope was
+        # called "NetworkScope".
+        addresses = [
+            {'NetworkName': '',
+             'NetworkScope': 'public',
+             'Type': 'hostname',
+             'Value': 'eu-west-1.compute.example.com'},
+            {'NetworkName': '',
+             'NetworkScope': 'local-cloud',
+             'Type': 'hostname',
+             'Value': 'eu-west-1.example.internal'},
+        ]
+        public_address = watchers.retrieve_public_adddress(addresses)
+        self.assertEqual('eu-west-1.compute.example.com', public_address)
+
     def test_last_unknown_address(self):
         # If the scope of multiple addresses is unknown, the last one is taken.
         addresses = [
             {'NetworkName': '',
-             'NetworkScope': '',
+             'Scope': '',
              'Type': 'ipv4',
              'Value': '10.0.3.42'},
             {'NetworkName': '',
-             'NetworkScope': '',
+             'Scope': '',
              'Type': 'ipv4',
              'Value': '10.0.3.47'},
         ]
